import { world, system } from "@minecraft/server";import { 
    getScore, 
    getPlayerCountWithTag, 
    playSound, Tellraw, 
    ActionBar, setScore,
    removeTagsByPrefix, 
    getPlayerNamesWithTag, 
    ReplaceItemHotbar, 
    getHealthPercentage, 
    ReplaceItem,
    hasItemInInventory,
    incrementScore,
    modifyScore,
    getRandomInRange,
    cloceForm, runcmd_at, runcmd

} from '../other/utils';
import { handleVictory } from "../game/vinerRaund";
import { bombCondition } from "../game/bomb";
import { UIupdateGame } from "../ui/globalGame";


const DEFUSE_TIME_NORMAL = 200;
const DEFUSE_TIME_KIT = 100;
const MOVEMENT_THRESHOLD = 1;
const VIEW_THRESHOLD = 0.6;
const DEFUSE_SYMBOLS = ["", "", "", "", "", "", ""];
const DEFUSE_SYMBOLS_KIT = ["", "", "", "", "", "", ""];
const COMPLETE_SYMBOL = "";

const defuseTimers = new Map();
const playerStartPositions = new Map();

world.beforeEvents.playerInteractWithEntity.subscribe(handleEntityInteraction);

export function handleEntityInteraction(event) {
    const player = event.player;
    const target = event.target;
    if (target.typeId !== "fear:entites_bomb") { 
        
        return; 
    }
    
        console.log(`"Interaction bomb111" ${player.name}`)

    if (!player) { 
        return; 
    }

    if (!player.hasTag("game:commando")) {
        return; }


    const bombStatus = getScore("fear:settings", "bomb_condition", true);
    if (bombStatus !== 1) {return;}

    const bomb = target;
    const hasKit = playerHasDefuseKit(player);
    playerStartPositions.set(player.id, { ...player.location });

    startDefuseTimer(player, bomb, hasKit);
}



function playerHasDefuseKit(player) {
    const inventory = player.getComponent("minecraft:inventory");
    if (!inventory) return false;

    const container = inventory.container;
    for (let i = 0; i < container.size; i++) {
        const item = container.getItem(i);
        if (item && item.typeId === "fear:sapper_kit") {
            return true;
        }
    }
    return false;
}

export function removeDefuseKit(player) {
    const name = player.name;
    runcmd_at(name, "clear @s fear:sapper_kit", true);
}

export function startDefuseTimer(player, bomb, hasKit) {
    if (defuseTimers.has(player.id)) return;

    const defuseTime = hasKit ? DEFUSE_TIME_KIT : DEFUSE_TIME_NORMAL;
    defuseTimers.set(player.id, 0);
    const name = player.name;

    runcmd_at(name, `execute as @s[hasitem={item=fear:sapper_kit, quantity=0}] at @s run playanimation @e[type=fear:entites_bomb] animation.cs_bomb.defused_password`, true);
    runcmd_at(name, `execute as @s[hasitem={item=fear:sapper_kit, quantity=1}] at @s run playanimation @e[type=fear:entites_bomb] animation.cs_bomb.defused_password_fast`, true);
    runcmd('execute as @e[type=fear:entites_bomb] at @s run execute as @a at @s run playsound fear.bomb_activation_sound @s ~~~ 0.7 1', true);

    world.getPlayers().forEach(p => {
        if (p !== player && distance(p.location, player.location) <= 15 && p.hasTag("game:commando")) {
            system.run(() => {
                p.playSound("fear.im_defusing_bomb", p.location);
            });
        }
    });

    function tick() {
        if (!defuseTimers.has(player.id)) return;

        const progress = defuseTimers.get(player.id) + 1;
        defuseTimers.set(player.id, progress);

        if (!isPlayerEligibleToDefuse(player, bomb)) {
            resetDefuseTimer(player);
            return;
        }

        setScore("fear:settings", "bomb_animation", defuseTime);

        const symbolIndex = Math.floor((progress / defuseTime) * DEFUSE_SYMBOLS.length);
        const symbol = hasKit ? DEFUSE_SYMBOLS_KIT[symbolIndex] : DEFUSE_SYMBOLS[symbolIndex];

        runcmd_at(name, `titleraw @s title {"rawtext":[{"text":"big_actionbar"},{"text":"${symbol} "},{"translate":"defuse.started"}]}`, true);

        if (progress >= defuseTime) {
            completeDefuse(player, hasKit);
        } else {
            system.runTimeout(tick, 1);
        }
    }

    tick();
}

export function isPlayerEligibleToDefuse(player, bomb) {
    const name = player.name;
    const startPos = playerStartPositions.get(player.id);
    const dx = Math.abs(player.location.x - startPos.x);
    const dz = Math.abs(player.location.z - startPos.z);

    if (dx > MOVEMENT_THRESHOLD || dz > MOVEMENT_THRESHOLD) {
        runcmd_at(name, `titleraw @s title {"rawtext":[{"text":"big_actionbar"},{"text":" §c"},{"translate":"defuse.interrupted"}]}`, true);

        bigActionbarClear(player);

        return false;
    }

    const viewDir = player.getViewDirection();
    const bombPos = bomb.location;
    const toBomb = {
        x: bombPos.x - player.location.x,
        y: bombPos.y - (player.location.y + 1.62),
        z: bombPos.z - player.location.z
    };

    const magnitude = Math.sqrt(toBomb.x ** 2 + toBomb.y ** 2 + toBomb.z ** 2);
    toBomb.x /= magnitude;
    toBomb.y /= magnitude;
    toBomb.z /= magnitude;

    const dotProduct = toBomb.x * viewDir.x + toBomb.y * viewDir.y + toBomb.z * viewDir.z;
    if (dotProduct < VIEW_THRESHOLD) {
        runcmd_at(name, `titleraw @s title {"rawtext":[{"text":"big_actionbar"},{"text":" §c"},{"translate":"defuse.interrupted"}]}`, true);

        bigActionbarClear(player);

        return false;
    }

    return true;
}

function resetDefuseTimer(player) {
    setScore("fear:settings", "bomb_animation", 0);
    if (defuseTimers.has(player.id)) {
        defuseTimers.delete(player.id);
        playerStartPositions.delete(player.id);
    }
}

export function completeDefuse(player, usedKit) {
    const name = player.name;

    setScore("fear:settings", "bomb_animation", 0);
    defuseTimers.delete(player.id);
    playerStartPositions.delete(player.id);

    if (usedKit) {
        removeDefuseKit(player);
    }

    setScore("fear:settings", "bomb_condition", 2);
    afterBombing();

    runcmd_at(name, `titleraw @s title {"rawtext":[{"text":"big_actionbar"},{"text":" §a"},{"translate":"defuse.completed"}]}`, true);
    bigActionbarClear(player);

    // Награды
    Tellraw(player, `+300$ §a`, "reward.defuse");
    modifyScore("fear:balance", name, 300);

    const counterTerrorists = world.getPlayers().filter(p => p.hasTag("game:commando"));
    counterTerrorists.forEach(ct => {
        modifyScore("fear:balance", ct.name, 3500);
        Tellraw(ct, "+$3500 §a", "reward.defuse_victory");
    });

    const playersInGame = world.getPlayers().filter(p => p.hasTag("fear:game"));
    playersInGame.forEach(p => {
        p.playSound("fear.bomb_deactivated", p.location);
    });

    system.runTimeout(() => {
        handleVictory("commando", playersInGame);
        UIupdateGame();
    }, 1);
    system.runTimeout(() => {
        UIupdateGame();
    }, 10);
}

function distance(pos1, pos2) {
    return Math.sqrt(
        (pos1.x - pos2.x) ** 2 +
        (pos1.y - pos2.y) ** 2 +
        (pos1.z - pos2.z) ** 2
    );
}

function afterBombing() {
    const interval = system.runInterval(() => {
        bombCondition()
        system.clearRun(interval);
    }, 50);
}

export function bigActionbarClear(player) {
    const name = player.name
    system.runTimeout(() => {
        runcmd_at(name, `title @s title big_actionbar`, true);
        system.runTimeout(() => {
            runcmd_at(name, `title @s title big_actionbar`, true);
        }, 20);
    }, 10);
}