

import { shopItemsitemID } from '../game/raisingItems';
import { world, system, Player, EntityHealthComponent, EntityComponentTypes  } from "@minecraft/server";
import { ckeckItemInHand, hasItemInHand, getSmaill,  } from "../game/itemsInHand";
import { itemInHandM } from "../game/itemM"; 
import {  updateTagData } from '../other/tagUtils';

import { 
    getScore, eventEntity,
    getPlayerCountWithTag, 
    playSound, Tellraw, 
    ActionBar, cloceUI,
    removeTagsByPrefix, 
    getPlayerNamesWithTag, 
    ReplaceItemHotbar, 
    getHealthPercentage, 
    ReplaceItem, getPlayerByName,
    hasItemInInventory,
    incrementScore,
    modifyScore,
    getRandomInRange,
    cloceForm, runcmd_at, runcmd

} from '../other/utils';


world.afterEvents.entityHurt.subscribe(eventData => {
    const hurtEntity = eventData.hurtEntity; 
    const damage = eventData.damage; 
    const damagingEntity = eventData.damageSource?.damagingEntity; 
    if (!hurtEntity || hurtEntity.typeId !== "fear:head") {
        return; 
    }

    const headOwnerTag = hurtEntity.nameTag;
    if (!headOwnerTag.startsWith("head:")) {
        return;
    }

    const playerName = headOwnerTag.replace("head:", "");
    const playerNameKiller = damagingEntity?.name;
    const ownerPlayer = getPlayerByName(playerName);

    if (!ownerPlayer) {
        return;
    }
    const gameMode = ownerPlayer.getGameMode(); 

    if (gameMode === "spectator") {
        return; 
    }
    const killer = getPlayerByName(playerNameKiller)
    if (hurtEntity.name === killer.name) {
        return;
    }
    if (!damagingEntity || damagingEntity.typeId !== "minecraft:player") {
        return;
    }
    if (damagingEntity.name === ownerPlayer.name) {
        return;
    }


    try {
        const healthComponent = ownerPlayer.getComponent("minecraft:health");
        if (!healthComponent) {
            return;
        }

        const increasedDamage = damage + 10;
        const currentHealth = healthComponent.currentValue;
        const predictedHealth = currentHealth - increasedDamage;

        if (predictedHealth <= 0) {
            let headshot = "";
            deachPlayer(ownerPlayer, killer, headshot);
            playSound(killer, "fear.headshot", 0.5, 1);
        }

        ownerPlayer.applyDamage(increasedDamage); 
        playSound(killer, "fear.hit_head", 0.5, 1);

    } catch (error) {
    }

    
    

});



export function isPlayerJumping(player) {
    try {
        const position = player.location;
        const dimension = player.dimension;
        const offsets = [
            { x: 0, z: 0 },
            { x: -0.3, z: -0.3 },
            { x: 0.3, z: 0.3 },
            { x: -0.3, z: 0.3 },
            { x: 0.3, z: -0.3 },
        ];

        for (const offset of offsets) {
            const blockBelow = dimension.getBlock({
                x: Math.floor(position.x + offset.x),
                y: Math.floor(position.y) - 1,
                z: Math.floor(position.z + offset.z),
            });

            if (blockBelow?.typeId === "gb:muzzle_flash2" || blockBelow?.typeId === "gb:muzzle_flash1") {
                return true; // Если нашли нужный блок — сразу true
            }
        }

        // Если ни один блок не подошел — только тогда false
        return false;
    } catch (error) {
        return false;
    }
}



export function deachPlayer(entity, killer, headshot = "") {
    const gameMode = entity.getGameMode();
    if (gameMode === "spectator") {
        return;
    }

    if (entity.hasTag(`game:deach`)) { 
        return;
    } else {
        entity.addTag("game:deach")
    }

    if (!entity || entity.typeId !== "minecraft:player") {
        return;
    }

    if (!killer || killer.typeId !== "minecraft:player") {
        return;
    }

    const playersInGame = world.getPlayers().filter(player => player.hasTag("fear:game"));


    let jump = "";

    if (isPlayerJumping(killer)) {
        jump = "";
    }


    
    let teamEntity = entity.hasTag("game:terrorist") ? "§c" : "§b";
    let teamKiller = killer.hasTag("game:terrorist") ? "§c" : "§b";
    let smoke = "";
    if (entity.hasTag(`game:in_smoke`)) { smoke = ""; }
    if (killer.hasTag(`game:in_smoke`)) { smoke = ""; }
    let flash = "";
    if (killer.hasTag(`game:in_flash`)) { flash = ""; }

    try {
            ckeckItemInHand(killer)
            .then(() => {
                const foundItem = hasItemInHand(killer);
                let scope = "";
                if ((foundItem === "sniper:awp" || foundItem === "sniper:barrett") && !killer.isSneaking) {
                    scope = "";
                }
                const assistant = `${flash}${scope}${smoke}${headshot}`;
                const itemData = itemInHandM.find(item => 
                    item.ID1 === foundItem || item.ID2 === foundItem || item.ID3 === foundItem
                );
                
                const rewardAmount = itemData && itemData.kill_reward ? parseInt(itemData.kill_reward) : 100;
                const itemSymbol = itemData ? itemData.displaySmaill : "";

                system.runTimeout(() => {
                    runcmd(`scoreboard players add @a[name="${killer.name}"] fear:balance ${rewardAmount}`, true);
                    runcmd(`scoreboard players add @a[name="${entity.name}"] fear:deach 1`, true);
                    runcmd(`scoreboard players add @a[name="${killer.name}"] fear:kill 1`, true);

                    updateTagData(killer, "kills", 1, true);
                    updateTagData(entity, "deaths", 1, true);


                    playersInGame.forEach(player => {
                        Tellraw(player, `${teamKiller}${killer.name} ${jump}${itemSymbol}${assistant} ${teamEntity}${entity.name}`);
                        
                    });
                    Tellraw(killer, `+${rewardAmount}$ §a`, "reward.kill");
                
                }, 10);
            })
            .catch(error => {
            });

    } catch (error) {
    }

    

    dropItemsOnDeath(entity, killer);
    
}



function dropItemsOnDeath(player, killer) {
    system.run(() => {
        const playerLocation = player.location;
        const inventory = player.getComponent("minecraft:inventory")?.container;
    
        if (player.hasTag("game:terrorist")) {
            runcmd(`summon fear:puppet ${playerLocation.x} ${playerLocation.y} ${playerLocation.z} facing @p[name="${killer.name}"] fear:puppet_terrorist "§c${player.name}"`, true);
        } else {
            runcmd(`summon fear:puppet ${playerLocation.x} ${playerLocation.y} ${playerLocation.z} facing @p[name="${killer.name}"] fear:puppet_commando "§b${player.name}"`, true);
        }
    
        if (!inventory) {
            return;
        } 
    
        
        const radius = 1;
        let angleIncrement = (2 * Math.PI) / shopItemsitemID.length;
    
        const hasBomb = (() => {
            for (let slot = 0; slot < inventory.size; slot++) {
                const slotItem = inventory.getItem(slot);
                if (slotItem && slotItem.typeId === "fear:bomb") {
                    return true;
                }
            }
            return false;
        })();
    
        if (hasBomb) {
            const bombX = playerLocation.x + radius * Math.cos(0); 
            const bombY = playerLocation.y + 0.5;
            const bombZ = playerLocation.z + radius * Math.sin(0);

            
            runcmd(`execute positioned ${playerLocation.x} ${playerLocation.y} ${playerLocation.z} run summon fear:entites_bomb ${bombX.toFixed(2)} ${bombY.toFixed(2)} ${bombZ.toFixed(2)} facing @p[name="${killer.name} fear:setIdTag"]`, true);
    
        }
    
        shopItemsitemID.forEach((item, index) => {
            if (["0", "1", "2"].includes(item.ID)) return;
    
            const itemIDsToCheck = [item.itemID, item.alternative1, item.alternative2].filter(Boolean);
            const foundItems = itemIDsToCheck.filter(itemID => {
                for (let slot = 0; slot < inventory.size; slot++) {
                    const slotItem = inventory.getItem(slot);
                    if (slotItem && slotItem.typeId === itemID) {
                        return true;
                    }
                }
                return false;
            });
    
            if (foundItems.length > 0) {
                const angle = index * angleIncrement;
    
                const spawnX = playerLocation.x + radius * Math.cos(angle);
                const spawnY = playerLocation.y + 0.5; 
                const spawnZ = playerLocation.z + radius * Math.sin(angle);
    
            
                runcmd(`execute positioned ${playerLocation.x} ${playerLocation.y} ${playerLocation.z} run summon fear:${item.name} ${spawnX.toFixed(2)} ${spawnY.toFixed(2)} ${spawnZ.toFixed(2)} facing @p fear:setIdTag`, true);
    
            }
        });
    
        runcmd(`clear @a[name="${player.name}"]`, true);
        runcmd(`gamemode spectator @a[name="${player.name}"]`, true);
        runcmd(`tag @a[name="${player.name}"] remove game:deach`, true);
    });
   
}


world.afterEvents.entityDie.subscribe(event => {
    const entity = event.deadEntity;
    if (!entity || entity.typeId !== "minecraft:player") return;
    const player = entity;
    system.run(() => {
        runcmd(`clear @a[name="${player.name}"]`, true);
        runcmd(`gamemode spectator @a[name="${player.name}"]`, true);
        runcmd(`tag @a[name="${player.name}"] remove game:deach`, true);
    });
});
