import { world, system } from '@minecraft/server';
import { 
  getScore, eventEntity,
  getPlayerCountWithTag, 
  playSound, Tellraw, 
  ActionBar, cloceUI,
  removeTagsByPrefix, 
  getPlayerNamesWithTag, 
  ReplaceItemHotbar, 
  getHealthPercentage, 
  ReplaceItem, getPlayerByName,
  hasItemInInventory,
  incrementScore,
  modifyScore,
  getRandomInRange,
  cloceForm
} from '../other/utils';

system.afterEvents.scriptEventReceive.subscribe(async (event) => {
  if (event.id === "fearcube:RadiusDamages") {
    (async () => {
      try {
        const playersInGame = world.getPlayers().filter(player => 
          player.hasTag("fear:game") && player.getGameMode() !== "spectator"
        );

        const [radiusStr, maxDamageStr] = event.message.split(',').map(str => str.split('=')[1]);
        const radius = parseInt(radiusStr, 10);
        const maxDamage = parseInt(maxDamageStr, 10);
        const entity = event.sourceEntity;
        const { x: sx, y: sy, z: sz } = entity.location;

        for (const player of playersInGame) {
          if (!player.hasTag('game:bomb_detonation')) continue;

          const { x: px, y: py, z: pz } = player.location;
          const distance = Math.sqrt((sx - px) ** 2 + (sy - py) ** 2 + (sz - pz) ** 2);

          if (distance <= radius) {
            const damageAmount = maxDamage * (1 - distance / radius);
            const roundedDamageAmount = Math.round(damageAmount);
            player.applyDamage(damageAmount);

            const healthComponent = player.getComponent('minecraft:health');
            let teamC = "", teamT = "";

            if (player.hasTag(`game:terrorist`)) { teamC = "§c"; teamT = ""; }
            if (player.hasTag(`game:commando`)) { teamC = ""; teamT = "§9"; }

            if (healthComponent && healthComponent.currentValue <= 0) {
              player.setGameMode("spectator");

              for (const p of playersInGame) {
                system.runTimeout(() => {
                  Tellraw(p, ` ${teamC}${teamT}${player.name} `);
                }, 10);
                
              }
            } else {
              for (const p of playersInGame) {
                system.runTimeout(() => {
                  Tellraw(p, ` ${teamC}${teamT}${player.name}  §c-${roundedDamageAmount}`);
                }, 10);
                
              }
            }
          }
        }

      } catch (error) {
        console.error(`[RadiusDamages Error]: ${error}`);
      }
    })();
  }
});
