import { world, system } from '@minecraft/server';
import { 
    getScore, setScore,
    getPlayerCountWithTag, 
    playSound, Tellraw, 
    ActionBar, cloceUI,
    removeTagsByPrefix, 
    getPlayerNamesWithTag, 
    ReplaceItemHotbar, 
    getHealthPercentage, 
    ReplaceItem,
    hasItemInInventory,
    incrementScore,
    modifyScore,
    getRandomInRange,
    cloceForm, runcmd_at, runcmd

} from '../other/utils';
import { bigActionbarClear } from '../game/bomb_defuse';

export function exchangeLobbyTag() {
    const terroristPlayers = world.getPlayers().filter(player => player.hasTag("lobby:terrorist"));
    terroristPlayers.forEach((player) => {
        cloceUI(player);
        removeTagsByPrefix(player, "lobby:"); 
        removeTagsByPrefix(player, "game:"); 
        removeTagsByPrefix(player, "fear:");
        removeTagsByPrefix(player, "commando:");
        player.addTag("game:terrorist"); 
        player.addTag("fear:game");
        runcmd_at(player.name, `scoreboard players set @s fear:balance 900`, true);
        runcmd(`event entity @e[type=fear:head] fear:despawn`, true);
    });

    const commandoPlayers = world.getPlayers().filter(player => player.hasTag("lobby:commando"));
    commandoPlayers.forEach((player) => {
        cloceUI(player);
        removeTagsByPrefix(player, "game:"); 
        removeTagsByPrefix(player, "lobby:");
        removeTagsByPrefix(player, "fear:");
        removeTagsByPrefix(player, "terrorist:");
        player.addTag("game:commando"); 
        player.addTag("fear:game");
        runcmd_at(player.name, `scoreboard players set @s fear:balance 900`, true);

    });
}



function tagBindingGameID(player) {
    removeGameIDTags(player)
    const gameID = getScore('fear:settings','gameID',true)
    player.addTag(`gameID:${gameID}`);
}

function removeGameIDTags(player) {
    const tags = player.getTags();
    const gameIDTags = tags.filter(tag => tag.startsWith("gameID:"));
    gameIDTags.forEach(tag => {
        player.removeTag(tag);
    });
}



export function assignTerroristTags() {
    const terroristPlayers = world.getPlayers()
        .filter(player => player.hasTag("game:terrorist") && !player.hasTag("game:commando"));

    const maxPlayers = 12;
    const delayStep = 5;

    terroristPlayers.slice(0, maxPlayers).forEach((player, index) => {
        const tag = `terrorist:${index + 1}`;

        system.runTimeout(() => {
            player.getTags()
                .filter(t => t.startsWith("terrorist:"))
                .forEach(t => player.removeTag(t));

            if (!player.getTags().includes(tag)) {
                player.addTag(tag);
                tagBindingGameID(player);

                const { x, y, z } = player.location;
                const name = player.name;

                runcmd_at(name, `summon fear:head "head:${name}" ${x} ${y + 2} ${z}`, true);
                runcmd_at(name, `tag @e[name="head:${name}"] add game:terrorist`, true);
                runcmd_at(name, `event entity @s fear:skin.terrorist`, true);
            }
        }, index * delayStep);
    });
}


export function assignCommandoTags() {
    const commandoPlayers = world.getPlayers()
        .filter(player => player.hasTag("game:commando") && !player.hasTag("game:terrorist"));

    const maxPlayers = 12;
    const delayStep = 5;

    commandoPlayers.slice(0, maxPlayers).forEach((player, index) => {
        const tag = `commando:${index + 1}`;

        system.runTimeout(() => {
            player.getTags()
                .filter(t => t.startsWith("commando:"))
                .forEach(t => player.removeTag(t));

            if (!player.getTags().includes(tag)) {
                player.addTag(tag);
                tagBindingGameID(player);

                const { x, y, z } = player.location;
                const name = player.name;

                runcmd_at(name, `summon fear:head "head:${name}" ${x} ${y + 2} ${z}`, true);
                runcmd_at(name, `tag @e[name="head:${name}"] add game:commando`, true);
                runcmd_at(name, `event entity @s fear:skin.commando`, true);
            }
        }, index * delayStep);
    });
}






const teleportTerroristLocation1 = [
    "1063 117 -21","1063 117 -23","1063 117 -19","1061 117 -26","1061 117 -16","1062 117 -17","1062 117 -25","1060 117 -25","1060 117 -17","1059 117 -23","1059 117 -19","1059 117 -21"
];
const teleportCommandoLocation1 = [
    "996 115 7","996 115 9","996 115 5","997 115 3","997 115 11","998 115 12","999 115 11","1000 115 9","1000 115 7","1000 115 5","999 115 3","998 115 2"
];



export function teleportPlayers() {
    const gameLocation = getScore('fear:settings', 'gameLocation', true);
    if (gameLocation !== 1) return;

    const terroristPlayers = world.getPlayers().filter(p => p.hasTag("game:terrorist"));
    const commandoPlayers = world.getPlayers().filter(p => p.hasTag("game:commando"));
    const playersInGame = world.getPlayers().filter(p => p.hasTag("fear:game"));
    playersInGame.forEach((player) => { 
        if (player.hasTag(`game:terrorist`) || player.hasTag(`game:commando`)) {
            runcmd_at(player.name, `inputpermission set @s movement disabled`, true);
        }
        
    })
    let delay = 0;

    for (let i = 0; i < terroristPlayers.length; i++) {
        if (i < teleportTerroristLocation1.length) {
            const [x, y, z] = teleportTerroristLocation1[i].split(" ").map(Number);
            const player = terroristPlayers[i];

            system.runTimeout(() => {
                player.teleport({ x, y, z }, world.getDimension("overworld"));
                player.setGameMode("adventure");
                bigActionbarClear(player);
            }, delay);
            
            delay += 2; 
        }
    }

    for (let i = 0; i < commandoPlayers.length; i++) {
        if (i < teleportCommandoLocation1.length) {
            const [x, y, z] = teleportCommandoLocation1[i].split(" ").map(Number);
            const player = commandoPlayers[i];

            system.runTimeout(() => {
                player.teleport({ x, y, z }, world.getDimension("overworld"));
                player.setGameMode("adventure");
                bigActionbarClear(player);
            }, delay);

            delay += 2; 
        }
    }
}
export function spawnpointPlayers() {
    const gameLocation = getScore('fear:settings', 'gameLocation', true);
    if (gameLocation !== 1) return;

    const terroristPlayers = world.getPlayers().filter(p => p.hasTag("game:terrorist"));
    const commandoPlayers = world.getPlayers().filter(p => p.hasTag("game:commando"));

    let delay = 0; 

    for (let i = 0; i < terroristPlayers.length; i++) {
        if (i < teleportTerroristLocation1.length) {
            const [x, y, z] = teleportTerroristLocation1[i].split(" ").map(Number);
            const player = terroristPlayers[i];

            system.runTimeout(() => {
                runcmd_at(player.name, `spawnpoint @s ${x} ${y} ${z}`, true);
            }, delay);

            delay += 2;
        }
    }

    for (let i = 0; i < commandoPlayers.length; i++) {
        if (i < teleportCommandoLocation1.length) {
            const [x, y, z] = teleportCommandoLocation1[i].split(" ").map(Number);
            const player = commandoPlayers[i];

            system.runTimeout(() => {
                runcmd_at(player.name, `spawnpoint @s ${x} ${y} ${z}`, true);
            }, delay);

            delay += 2;
        }
    }
}



