import { system, world, ItemStack  } from "@minecraft/server";
import { UIupdateGame } from '../ui/globalGame';
import { startTimer } from '../game/purchaseTimer';
import { teleportPlayers } from '../game/distribution';
import { checkVictoryConditions } from '../game/vinerRaund';
import { dropBombExecute, bombCondition } from '../game/bomb';
import { teleportHeadsToPlayers } from '../game/customHead';
import { setAnimationsPlant } from '../game/startGame';
import { handleVictory } from '../game/vinerRaund';
import { hasItemInHand, ckeckItemInHand} from '../game/itemsInHand';
import { 
    getScore, 
    getPlayerCountWithTag, 
    playSound, Tellraw, 
    ActionBar, 
    removeTagsByPrefix, 
    getPlayerNamesWithTag, 
    ReplaceItemHotbar, 
    getHealthPercentage, 
    ReplaceItem,
    hasItemInInventory,
    incrementScore,
    modifyScore,
    getRandomInRange,
    cloceForm,
    setScore, runcmd_at, runcmd

} from '../other/utils';


let activeInterval = null;

export function startTimerExplode() {
    if (activeInterval) {
        console.warn("startTimerExplode уже запущен, повторный запуск отменён");
        return; 
    }

    const playersInGame = world.getPlayers().filter(player => player.hasTag("fear:game"));
    const bombEntities = world.getDimension("overworld").getEntities().filter(entity => entity.typeId === "fear:entites_bomb");

    const startTimerExplode = getScore('fear:settings', 'startTimerExplode', true);
    if (startTimerExplode === 0) {
        setScore("fear:settings", "startTimerExplode", 45);
    }

    
    setScore("fear:settings", "restarter_map", 4);

    let remaining = getScore('fear:settings', 'startTimerExplode', true);
    if (remaining !== null && remaining > 0 && activeInterval !== null) {
        return; 
    }

    activeInterval = system.runInterval(() => {
        teleportHeadsToPlayers();
    }, 1);

    activeInterval = system.runInterval(() => {
        setAnimationsPlant();
    }, 100);

    activeInterval = system.runInterval(() => {
        remaining = getScore('fear:settings', 'startTimerExplode', true);
        bombCondition()

        if (bombEntities.length > 0) {
            if (remaining > 10) {
                bombEntities.forEach((entity) => { 
                    runcmd('execute as @e[type=fear:entites_bomb] at @s run execute as @a[r=20] at @s run playsound fear.bomb_sound @s ~~~ 1 1', true);
                    runcmd('execute as @e[type=fear:entites_bomb] at @s run execute as @a[rm=10,r=20] at @s run playsound fear.bomb_sound @s ~~~ 0.5 1', true);
                    runcmd('execute as @e[type=fear:entites_bomb] at @s run execute as @a[rm=20,r=40] at @s run playsound fear.bomb_sound @s ~~~ 0.2 1', true);
                    UIupdateGame();
                });
            }
            
        }

        if (remaining === 11) {
            timerExplodeSound( 10, 10)
            
        }
        if (remaining === 6) {
            timerExplodeSound( 10, 6)
            
        }
        if (remaining === 3) {
            timerExplodeSound( 10, 4)
            
        }

        if (remaining === 1 && bombEntities.length > 0) {
            bombEntities.forEach((entity) => { 
                runcmd('execute as @e[type=fear:entites_bomb] at @s run playsound fear.nvg_on @a ~~~', true);
                UIupdateGame();
            });
        }

        
        if (remaining > 0) {
            checkVictoryConditions();
            dropBombExecute();
            modifyScore("fear:settings", "startTimerExplode", -1);
        } else {
            const restarter_map = getScore('fear:settings', 'restarter_map', true);
            if (restarter_map === 4) {
                if (bombEntities.length > 0) {
                    const terrorists = world.getPlayers().filter(p => p.hasTag("game:terrorist"));
                        terrorists.forEach(t => {
                            modifyScore("fear:balance", t.name, 3500);
                            Tellraw(t, "+$3500 §a", "reward.explosion_victory");
                        });
    
                    runcmd('execute as @e[type=fear:entites_bomb] at @s run playsound fear.bomb_explosion @a ~~~', true);
                    runcmd('execute as @a[tag=fear:game] at @s run playsound fear.bomb_explosion @a ~~~ 0.3 1', true);
                    runcmd('execute as @e[type=fear:entites_bomb] at @s run particle minecraft:huge_explosion_lab_misc_emitter ~~~', true);
                    runcmd('execute as @e[type=fear:entites_bomb] at @s run tag @a[tag=fear:game,m=adventure,r=45] add game:bomb_detonation', true);
                    runcmd('execute as @e[type=fear:entites_bomb] at @s run scriptevent fearcube:RadiusDamages r=45,maxDamage=30', true);
    
                }
    
                handleVictory("terrorist", playersInGame);
                
                stopTimerExplode();  
                system.runTimeout(() => {
                    setScore("fear:settings", "purchaseTimer", 20);
                    startTimer(); 
                }, 150);
                system.runTimeout(() => {
                    teleportPlayers();
                }, 145);

            }
        }
        playersInGame.forEach(player => {
                    ckeckItemInHand(player)
                    .then(() => {
                        const foundItem = hasItemInHand(player);
                        if ((foundItem === "addon:combat_knife")) {
                            runcmd_at(player.name, `event entity @s normal:player`, true);
                        }
                    })
                });
    }, 20);
}

function timerExplodeSound(countSounds, speedSound) {
    let soundCounter = 0;

    const soundInterval = system.runInterval(() => {
        const bombStatus = getScore("fear:settings", "bomb_condition", true);
        if (bombStatus !== 1) {
            system.clearRun(soundInterval);
            return;
        }
        if (soundCounter >= countSounds) {
            system.clearRun(soundInterval);
            return;
        }

        runcmd('execute as @e[type=fear:entites_bomb] at @s run execute as @a[r=20] at @s run playsound fear.bomb_sound @s ~~~ 1 1', true);
        runcmd('execute as @e[type=fear:entites_bomb] at @s run execute as @a[rm=10,r=20] at @s run playsound fear.bomb_sound @s ~~~ 0.7 1', true);
        runcmd('execute as @e[type=fear:entites_bomb] at @s run execute as @a[rm=20,r=40] at @s run playsound fear.bomb_sound @s ~~~ 0.2 1', true);

        UIupdateGame();

        soundCounter++;
    }, speedSound);
}


export function stopTimerExplode() {
    setScore("fear:settings", "startTimerExplode", 0);

    if (activeInterval !== null) {
        system.clearRun(activeInterval);
        activeInterval = null;
    }
}
