import { world, system } from "@minecraft/server";
import { getPlayerTeam, distance } from "./grenades.js";
import {  Tellraw, modifyScore, runcmd_at, runcmd } from '../../other/utils.js';

const activeSmokeGrenades = new Map();

world.beforeEvents.itemUse.subscribe(event => {
    const { itemStack, source } = event;
    
    if (itemStack.typeId === "addon:smoke_grenade" && source.typeId === "minecraft:player") {
        activeSmokeGrenades.set(source.id, {
            name: source.name,
            team: getPlayerTeam(source),
            location: { ...source.location }
        });
    }
});

export function handleSmokeGrenade(entity) {
    let thrower = getNearestThrower(entity.location);
    if (!thrower) return;

    function checkFall() {
        if (!entity || entity.isRemoved) return;

        if (entity.getVelocity().y === 0) {
            system.runTimeout(() => activateSmokeGrenade(entity), 10);
        } else {
            system.run(checkFall);
        }
    }

    system.run(checkFall);

    activeSmokeGrenades.delete(thrower.id);
}





function getNearestThrower(grenadePos) {
    let nearest = null;
    let minDist = 4;

    for (let [id, thrower] of activeSmokeGrenades.entries()) {
        let dist = distance(grenadePos, thrower.location);
        if (dist < minDist) {
            minDist = dist;
            nearest = { id, ...thrower };
        }
    }
    return nearest;
}


export function activateSmokeGrenade(grenade) {
    if (!grenade || grenade.isRemoved) return;

    const dim = world.getDimension("overworld");
    dim.playSound("addon.smoke_grenade_explode", grenade.location);

    let lifeTime = 400;
    let tickCounter = 0;
    let smokePaused = false;

    function spawnSmoke() {
        if (!grenade || grenade.isRemoved) return;
        if (tickCounter >= lifeTime) {
            grenade.remove();
            return;
        }

        if (grenade.hasTag("game:smokFalse")) {
            if (!smokePaused) {
                smokePaused = true;
                spawnSmokeParticles2(grenade.location);

                system.runTimeout(() => {
                    smokePaused = false;

                    runcmd(`execute as @e[type=addon:smoke_grenade] at @s run tag @s remove game:smokFalse`, true);
                    dim.playSound("fear.smoke_grenade_explode2", grenade.location);
                }, 40);
            }
        }

        if (!smokePaused) {
            spawnSmokeParticles(grenade.location);
        }

        runcmd(`execute as @e[type=addon:smoke_grenade] at @s run event entity @e[r=4,type=addon:fire_grenade,tag=game:grenadeFireTrue] entity:despawn`, true);

        for (const player of world.getAllPlayers()) {
            const dist = distance(grenade.location, player.location);

            if (dist <= 2.5) {
                const playerName = player.name;
                runcmd_at(playerName, "camera @s fade time 0.1 0 0.1 color 60 60 60", true);
                runcmd_at(playerName, "tag @s add game:in_smoke", true);

                system.runTimeout(() => {
                    runcmd_at(playerName, "tag @s remove game:in_smoke", true);
                }, 40);
            }
        }

        tickCounter += 20;
        system.runTimeout(spawnSmoke, 20);
    }

    spawnSmoke();
}


function spawnSmokeParticles(location) {
    const offsets = [
        [0, 0, 0], [0.4, 0, 0], [-0.4, 0, 0], [-0.4, 0, -0.4], [0.4, 0, 0.4],
        [-0.4, 0, 0.4], [0.4, 0, -0.4], [1, 0, 0], [-1, 0, 0], [-1, 0, -1],
        [1, 0, 1], [-1, 0, 1], [1, 0, -1], [1.5, 0, -1.5], [-1.5, 0, 1.5],
        [1, 1, 0], [-1.5, 1, 0], [1.5, 1, 0], [0, 1, 1.5], [0, 1, -1.5]
    ];

    for (let [dx, dy, dz] of offsets) {
        world.getDimension("overworld").spawnParticle("addon:smoke_grenade", {
            x: location.x + dx,
            y: location.y + dy,
            z: location.z + dz
        });
    }
}

function spawnSmokeParticles2(location) {
    const offsets = [
        [0, 0, 0], [0.4, 0, 0], [-0.4, 0, 0]
    ];

    for (let [dx, dy, dz] of offsets) {
        world.getDimension("overworld").spawnParticle("addon:smoke_grenade", {
            x: location.x + dx,
            y: location.y + dy,
            z: location.z + dz
        });
    }
}