import { system, world } from "@minecraft/server";

import { itemInHandM } from "../game/itemM";
import { runcmd_at, runcmd } from "../other/utils";

// Получаем читаемое имя предмета
export function getReadableName(itemId) {
    const itemConfig = itemInHandM.find(item => 
        item.ID1 === itemId || item.ID2 === itemId || item.ID3 === itemId
    );

    return itemConfig ? itemConfig.displayText : "Неизвестный предмет";
}


// Получаем смайлик
export function getSmaill(itemId) {
    const itemConfig = itemInHandM.find(item => 
        item.ID1 === itemId || item.ID2 === itemId || item.ID3 === itemId
    );

    return itemConfig ? itemConfig.displaySmaill : "";
}

// Проверяем наличие предмета в руке
export function ckeckItemInHand(player) {
    if (!player || !player.name) {
        console.error("Игрок не определён или имя игрока некорректно.");
        return Promise.reject(new Error("Игрок не определён."));
    }

    const promises = itemInHandM.flatMap(item => {
        const idsToCheck = [item.ID1, item.ID2, item.ID3].filter(Boolean);

        return idsToCheck.map(id => {
            const command = `execute as "${player.name}" if entity @s[hasitem={item=${id},location=slot.weapon.mainhand}] run tag @s add "item:${id}"`;

            try {
                return world.getDimension("overworld").runCommand(command);
            } catch (error) {
                console.error(`Ошибка выполнения команды: ${command}`, error);
                return Promise.reject(error);
            }
        });
    });

    return Promise.all(promises)
        .catch(error => console.error("Ошибка при проверке предметов:", error));
}



// Проверяем наличие тега предмета
export function hasItemInHand(player) {
    const tags = player.getTags();
    const gameIDTags = tags.filter(tag => tag.startsWith("item:"));

    if (gameIDTags.length > 0) {
        const foundTag = gameIDTags[0];
        const itemId = foundTag.replace("item:", "");

        // Удаляем все теги предметов
        gameIDTags.forEach(tag => {
            player.removeTag(tag);
        });

        return itemId;
    }

    return null;
}