import { system, world, ItemStack  } from "@minecraft/server";
import { UIupdateGame, clearLobbyUI } from '../ui/globalGame';
import { startTimerSpecialForces } from '../game/special_forces_timer';
import { giveBombToRandomTerrorist, dropBombExecute } from '../game/bomb';
import { setRedNickname, setBlueNickname, setInvisibleNickname } from '../other/playerNickName';
import { shopItemsitemID } from "../game/raisingItems"; 
import { overGame } from '../game/overGame';
import { 
    getScore, 
    getPlayerCountWithTag, 
    playSound, Tellraw, 
    ActionBar, setScore,
    removeTagsByPrefix, 
    getPlayerNamesWithTag, 
    ReplaceItemHotbar, 
    getHealthPercentage, 
    ReplaceItem,
    hasItemInInventory,
    incrementScore,
    modifyScore,
    getRandomInRange,
    cloceForm, runcmd, runcmd_at

} from '../other/utils';


let activeInterval = null;

export function timerPurchase() {
    startTimer()
}

export function startTimer() {
    if (activeInterval) {
        console.warn("startTimer уже запущен, повторный запуск отменён");
        return; 
    }
    const terrorists = world.getPlayers().filter(p => p.hasTag("game:terrorist"));
    const commandos = world.getPlayers().filter(p => p.hasTag("game:commando"));
    const roundsTakenTerrorist = getScore("fear:settings", "roundsTakenTerrorist", true);
    const roundsTakenCommando = getScore("fear:settings", "roundsTakenCommando", true);

    if (terrorists.length === 0 || commandos.length === 0) {
        const overGameStatus = 0;
        overGame(overGameStatus); 
        return; 
    }
    if (roundsTakenTerrorist >= 8 || roundsTakenCommando >= 8) {
        const overGameStatus = 1;
        overGame(overGameStatus); 
        return; 
    }


    const purchaseTimer = getScore('fear:settings', 'purchaseTimer', true);
    if (purchaseTimer === 0) {
        setScore("fear:settings", "purchaseTimer", 20);
    }

    const playersInGame = world.getPlayers().filter(player => player.hasTag("fear:game"));
    playersInGame.forEach((player) => {
        const name = player.name;
        runcmd_at(name, `title @s title big_actionbar`, true);
        runcmd_at(name, `title @s title linen_plate`, true);
        runcmd_at(name, `title @s title scoreboard`, true);
    });

    setScore("fear:settings", "bomb_condition", 0);
    setScore("fear:settings", "restarter_map", 2);
    giveBombToRandomTerrorist();

    runcmd('kill @e[type=item]', true);
    runcmd('event entity @e[type=addon:fire_grenade] entity:despawn', true);
    runcmd('event entity @e[family=cs_drop] fear:despawn', true);
    playersInGame.forEach((player) => {
        const name = player.name;
    

        system.runTimeout(() => {
            AutomaticReloadingCartridges(player);
        }, 200);

        system.runTimeout(() => {
            reloadWeapons(player);
        }, 100);

        system.runTimeout(() => {
            checkAndLimitBalance(player);
        }, 10);

        system.runTimeout(() => {
            ReplaceItemHotbar(player, "fear:shoop", 8, 1, true);
            ReplaceItemHotbar(player, "addon:combat_knife", 0, 1, true);
            ReplaceItem(player, "pistol:taurus", "slot.hotbar", 1, 1, true);
            ReplaceItem(player, "pistol:taurus_mag", "slot.inventory", 0, 64, true);
        }, 4);

        system.runTimeout(() => {
            player.triggerEvent("fear:custom_hitbox_off");
            runcmd_at(name, `effect @s instant_health 1 100 true`, true);
        }, 5);

        player.removeTag("game:bomb_detonation");

        if (player.hasTag("game:terrorist")) {
            setRedNickname(player);
        } else {
            setBlueNickname(player);
        }
    });

    let remaining = getScore('fear:settings', 'purchaseTimer', true);
    if (remaining !== null && remaining > 0 && activeInterval !== null) {
        return;
    }

    activeInterval = system.runInterval(() => {
        remaining = getScore('fear:settings', 'purchaseTimer', true);

        if (remaining > 0) {
            UIupdateGame();
            dropBombExecute();
            modifyScore("fear:settings", "purchaseTimer", -1);

        } else if (remaining === 1) {
            playersInGame.forEach((player) => {
                
                clearLobbyUI(player);
                if (player.hasTag("game:terrorist")) {
                    setRedNickname(player);
                } else if (player.hasTag("game:commando")) {
                    setBlueNickname(player);
                }
            });

        } else {
            const restarter_map = getScore('fear:settings', 'restarter_map', true);
            if (restarter_map === 2) {
                world.getPlayers().filter(p => p.hasTag("fear:game")).forEach((player) => {
                    try {
                        const name = player.name;
                        const sound = getRandomInRange(1, 2);
        
                        cloceForm(player);
                        runcmd_at(name, `inputpermission set @s movement enabled`, true);
                        runcmd_at(name, `clear @s fear:shoop`, true);
                        player.triggerEvent("fear:custom_hitbox_on");
                        setInvisibleNickname(player);
        
                        if (sound === 1) {
                            playSound(player, "fear.okay_lets_go", 1, 1);
                        } else {
                            playSound(player, "fear.go_go_go", 1, 1);
                        }
                    } catch (err) {
                    }
                });

    
                stopTimer();
                setScore("fear:settings", "drawVictory", 0);
                setScore("fear:settings", "startTimerSpecialForces", 115);
                startTimerSpecialForces();
                setScore("fear:settings", "bomb_protection", 0);
                setScore("fear:settings", "double_protection", 0);
                
            }

        }
    }, 20);
}

export function stopTimer() {
    setScore("fear:settings", "purchaseTimer", 0);
    if (activeInterval !== null) {
        system.clearRun(activeInterval);
        activeInterval = null;
    }
}

function checkAndLimitBalance(player) {
    const maxBalance = 16000;
    const currentBalance = getScore("fear:balance", player.name, true);

    if (currentBalance > maxBalance) {
        setScore("fear:balance", player.name, maxBalance);
    }
}

export function AutomaticReloadingCartridges(player) {
    if (!player) return;

    const ammoData = [
        { gun: "taurus", ammo: 18 },
        { gun: "akm", ammo: 30 },
        { gun: "awp", ammo: 10 },
        { gun: "barrett", ammo: 5 },
        { gun: "m249", ammo: 100 },
        { gun: "m3", ammo: 8 },
        { gun: "mk18", ammo: 30 },
        { gun: "mp5", ammo: 30 },
        { gun: "p90", ammo: 50 }
    ];

    ammoData.forEach(({ gun, ammo }) => {
        runcmd(`scoreboard players set "${player.name}" "${gun}" ${ammo}`, true);
    });
}



export function reloadWeapons(player) {
    if (!player) return;

    const inventory = player.getComponent("minecraft:inventory");
    if (!inventory) return;

    const container = inventory.container;
    
    shopItemsitemID.forEach(itemConfig => {
        if (!itemConfig.alternative1 || !itemConfig.alternative2) return;

        for (let i = 0; i < container.size; i++) {
            const item = container.getItem(i);
            
            if (item && item.typeId === itemConfig.alternative1) { 
                ReplaceItem(
                    player,
                    itemConfig.itemID, 
                    itemConfig.slotID,
                    itemConfig.slot,
                    parseInt(itemConfig.quantity || 1),
                    true
                );

                ReplaceItem(
                    player,
                    itemConfig.itemID1, 
                    itemConfig.slotID1,
                    itemConfig.slot1,
                    parseInt(itemConfig.quantity1 || 1),
                    true
                );

                break; 
            }
        }
    });
}
