import { system, world, ItemStack  } from "@minecraft/server";
import { teleportPlayers } from '../game/distribution';
import { stopGameTimers } from '../game/overGame';
import { startTimer } from '../game/purchaseTimer';
import { stopTimerSpecialForces } from '../game/special_forces_timer';
import { overGame } from "../game/overGame.js";
import { 
    getScore, 
    getPlayerCountWithTag, 
    playSound, Tellraw, 
    ActionBar, 
    removeTagsByPrefix, 
    getPlayerNamesWithTag, 
    ReplaceItemHotbar, 
    getHealthPercentage, 
    ReplaceItem,
    hasItemInInventory,
    incrementScore,
    modifyScore,
    getRandomInRange,
    cloceForm,
    setScore, runcmd

} from '../other/utils';

export function checkPlayersAlive() {
    let overGameStatus = 0;
    const players = world.getPlayers();

    const terrorists = players.filter(player =>
        player.hasTag("game:terrorist")
    );

    const commandos = players.filter(player =>
        player.hasTag("game:commando")
    );

    if (terrorists.length === 0 || commandos.length === 0) {
        overGame(overGameStatus);
    }
}

export function checkVictoryConditions() {
    const players = world.getPlayers();
    const playersInGame = players.filter(player => player.hasTag("fear:game"));

    const terrorists = players.filter(player =>
        player.hasTag("game:terrorist") && player.getGameMode() === "adventure"
    );
    const commandos = players.filter(player =>
        player.hasTag("game:commando") && player.getGameMode() === "adventure"
    );

    const bombStatus = getScore("fear:settings", "bomb_condition", true); 

    if (bombStatus === 0) { 
        if (terrorists.length === 0 && commandos.length > 0) {
            handleVictory("commando", playersInGame);
            rewardPlayers("commando", playersInGame);
            
        } else if (commandos.length === 0 && terrorists.length > 0) {
            handleVictory("terrorist", playersInGame); 
            rewardPlayers("terrorist", playersInGame);

        } else if (commandos.length === 0 && terrorists.length === 0) {
            drawVictory()
        }
    } else if (bombStatus === 1) { 
        if (commandos.length === 0) {
            handleVictory("terrorist", playersInGame);
            rewardPlayers("terrorist", playersInGame);

        }
    }
}


function rewardPlayers(winningTeam, playersInGame) {

    playersInGame.forEach(player => {
        if ((winningTeam === "commando" && player.hasTag("game:commando")) || 
            (winningTeam === "terrorist" && player.hasTag("game:terrorist"))) {
            modifyScore("fear:balance", player.name, 3250);
            Tellraw(player, `+$3250 §a`, "reward.victory"); 
        }
    });
}

function drawVictory() {
    const drawVictory = getScore("fear:settings", "drawVictory", true);

    if (drawVictory === 0) {
        setScore("fear:settings", "drawVictory", 1);
        const players = world.getPlayers().filter(p => p.hasTag("fear:game"));
        players.forEach(p => {
            Tellraw(p, `+$0 §a`, "fearcube.game.draw");
            stopGameTimers();
            system.runTimeout(() => {
                setScore("fear:settings", "purchaseTimer", 20);
                startTimer();
            }, 150);
            system.runTimeout(() => {
                teleportPlayers();
            }, 145);
        });
    }
    
}

export function handleVictory(winningTeam, playersInGame) {
    const double_protection = getScore("fear:settings", "double_protection", true);
    if (double_protection === 1) {
        console.error("Отмена двойного выиграша")
        return; 
    }
    setScore("fear:settings", "double_protection", 1);
    
    system.runTimeout(() => {
        runcmd('event entity @e[family=cs_drop] fear:despawn', true);
    }, 140);
    const bombStatus = getScore("fear:settings", "bomb_condition", true);
    const sound = winningTeam === "commando" 
        ? "fear.counter_terrorists_win" 
        : "fear.terrorists_win";

    stopGameTimers(); 

    if (winningTeam === "commando") {
        modifyScore("fear:settings", "roundsTakenCommando", 1);
        modifyScore("fear:settings", "loss_in_rowT", 1);
        setScore("fear:settings", "loss_in_rowCT", 1);
    } else {
        modifyScore("fear:settings", "roundsTakenTerrorist", 1);
        modifyScore("fear:settings", "loss_in_rowCT", 1);
        setScore("fear:settings", "loss_in_rowT", 1);
    }

    const loss_in_rowCT = getScore("fear:settings", "loss_in_rowCT", true);
    const loss_in_rowT = getScore("fear:settings", "loss_in_rowT", true);

    system.runTimeout(() => {
        if (winningTeam === "commando") {
            const terrorists = world.getPlayers().filter(p => p.hasTag("game:terrorist"));
            giveLossBonus(terrorists, loss_in_rowT);
        } else {
            const counterTerrorists = world.getPlayers().filter(p => p.hasTag("game:commando"));
            giveLossBonus(counterTerrorists, loss_in_rowCT);
        }
        
    }, 10);


    if (bombStatus === 2) {
        system.runTimeout(() => {
            playersInGame.forEach(player => playSound(player, sound, 1, 1));
        }, 40);
    } else {
        playersInGame.forEach(player => playSound(player, sound, 1, 1));
    }

    system.runTimeout(() => {
        setScore("fear:settings", "purchaseTimer", 20);
        startTimer();
    }, 150);
    system.runTimeout(() => {
        teleportPlayers();
    }, 145);
}

function giveLossBonus(players, lossCount) {

    let bonusAmount = 0;
    switch (lossCount) {
        case 1: bonusAmount = 1400; break;
        case 2: bonusAmount = 1900; break;
        case 3: bonusAmount = 2400; break;
        case 4: bonusAmount = 2900; break;
        default: 
            if (lossCount >= 5) bonusAmount = 3400;
            break;
    }

    if (bonusAmount > 0) {
        players.forEach(player => {
            const gameMode = player.getGameMode?.() ?? player.getComponent("minecraft:game_mode")?.value;

            if (gameMode === "spectator") {
                system.runTimeout(() => {
                    modifyScore("fear:balance", player.name, bonusAmount);
                    Tellraw(player, `+$${bonusAmount} §a`, "reward.defeat");
                }, 10);
            }
        });
    }
}

