import { system, world } from '@minecraft/server';
import { 
    getScore, eventEntity,
    getPlayerCountWithTag, 
    playSound, Tellraw, setScore,
    ActionBar, cloceUI,
    removeTagsByPrefix, 
    getPlayerNamesWithTag, 
    ReplaceItemHotbar, 
    getHealthPercentage, 
    ReplaceItem, getPlayerByName,
    hasItemInInventory,
    incrementScore,
    modifyScore,
    getRandomInRange,
    cloceForm, runcmd, runcmd_at

} from '../other/utils';
import { UiLobby } from "../ui/glogalLobby.js";
import { startGame } from "../game/startGame.js";

let activeTimer = null;
let currentRemaining = null;

function stopTimer() {
    if (activeTimer !== null) {
        system.clearRun(activeTimer);
        activeTimer = null;
        currentRemaining = null;
        UiLobby()
    } 
}

function startTimer(duration) {
    
    const gameStatus = getScore('fear:settings', 'gameStatus', true);
    if (gameStatus === 1) {
        return; 
    }

    if (activeTimer !== null) {
        return; 
    }

    let remaining = duration; 
    setScore("fear:settings", "restarter_map", 1);
    currentRemaining = duration; 

    activeTimer = system.runInterval(() => {
        const teamT = getPlayerCountWithTag("lobby:terrorist");
        const teamC = getPlayerCountWithTag("lobby:commando");
        const players = world.getPlayers().filter(player => player.hasTag("fear:lobby"));

        if (teamT === 0 || teamC === 0) {
            stopTimer();
            return;
        }

        if (remaining > 0) {
            if (remaining === 30) {
                players.forEach(player => {
                    playSound(player, "note.xylophone", 1, 1);
                    ActionBar(player, "§7", `fearcube.title.timer.prefix`, ` §c${remaining}§7 `, `fearcube.title.timer.suffix`);
                });
            }
            if (remaining >= 6 && remaining <= 29) {
                players.forEach(player => playSound(player, `random.click`, 0.5, 1));
            }
            if (remaining < 30) { UiLobby() }
            if (remaining === 5) { 
                players.forEach(player => {
                    playSound(player, "note.xylophone", 1, 1);
                    ActionBar(player, "§7", `fearcube.title.timer.prefix`, ` §c${remaining}§7 `, `fearcube.title.timer.suffix`);
                });
            }
            if (remaining >= 1 && remaining <= 5) { 
                players.forEach(player => {
                    playSound(player, "random.click", 0.5, 1);
                    ActionBar(player, "§7", `fearcube.title.timer.prefix`, ` §c${remaining}§7 `, `fearcube.title.timer.suffix`);
                });
            }

            remaining--; 
            currentRemaining = remaining; 
            setScore("fear:settings", "lobby_timer", currentRemaining);
        } else {
            if (teamT >= 1 && teamC >= 1) {
                players.forEach(player => {
                    runcmd_at(player.name, `clear @s`, true);
                });
                stopTimer();
                startGame() 
            } else {
                stopTimer();
            }
        }
    }, 20);
}

export function startLobby() {
    const teamT = getPlayerCountWithTag("lobby:terrorist");
    const teamC = getPlayerCountWithTag("lobby:commando");


    if (activeTimer !== null) {
        return; 
    } else {
        if (teamT >= 1 && teamC >= 1) {
            startTimer(30);
        } else {
        }
    }

    if (teamT === 0 || teamC === 0) {
        stopTimer();
        return;
    }
}

export function getTimerValue() {
    return currentRemaining; 
}

export { startTimer, stopTimer };
