import { world, Player, system } from "@minecraft/server";
import { startLobby } from '../lobby/timer';
import { UiLobby } from "../ui/glogalLobby.js";
import { restoreOriginalNickname } from '../other/playerNickName';
import { 
    getScore, 
    getPlayerCountWithTag, 
    playSound, Tellraw, 
    ActionBar, cloceUI,
    removeTagsByPrefix, 
    getPlayerNamesWithTag, 
    ReplaceItemHotbar, 
    getHealthPercentage, 
    ReplaceItem,
    hasItemInInventory,
    incrementScore,
    modifyScore, clearNonPlayerTags,
    getRandomInRange,
    cloceForm, runcmd_at, runcmd

} from '../other/utils';


export function toLobby(player) {
    toCommands(player);
    cloceUI(player);
    restoreOriginalNickname(player);

    const centerX = 67;
    const centerZ = 82;
    const radius = 3;
    const fixedY = 33;
    const randomX = centerX + Math.floor((Math.random() - 0.5) * 2 * radius);
    const randomZ = centerZ + Math.floor((Math.random() - 0.5) * 2 * radius);
    const name = player.name;

    system.run(() => {
        try {
            player.triggerEvent("fear:custom_hitbox_off");
            player.triggerEvent("fear:skin.defalaut");

            runcmd_at(name, `tp @s ${randomX} ${fixedY} ${randomZ}`, true);
            RemoveAllTags(player, "lobby");
            runcmd_at(name, `inputpermission set @s movement enabled`, true);
            

        } catch (error) {
            console.warn(`Ошибка в toLobby для ${name}:`, error);
        }
    });
}


export function setLobby(player) {
    const centerX = 68; 
    const centerZ = 108;
    const radius = 3;
    const fixedY = 33;

    const randomX = centerX + Math.floor((Math.random() - 0.5) * 2 * radius);
    const randomZ = centerZ + Math.floor((Math.random() - 0.5) * 2 * radius);

    try {
        runcmd_at(player.name, `tp @s ${randomX} ${fixedY} ${randomZ}`, true);
        runcmd_at(player.name, `spawnpoint @s ${randomX} ${fixedY} ${randomZ}`, true);
    } catch (error) {
    }
    setCommands(player);
    cloceForm(player);
    cloceUI(player);
    restoreOriginalNickname(player);
    RemoveAllTags(player, "hub");
    
    ReplaceItemHotbar(player, "fear:map_information", 8, 1, true);
    ReplaceItemHotbar(player, "fear:playgame", 0, 1, true);

    system.run(() => {
        player.triggerEvent("fear:custom_hitbox_off");
        player.triggerEvent("fear:skin.defalaut");
        player.setGameMode("adventure");
    });
 
    system.runTimeout(() => {
        cloceUI(player);
    }, 20);
}

function setCommands(player) {
    const commands = [
        `tag @s remove fear:lobby`,
        `event entity @s fear:skin.defalaut`,
        `inputpermission set @s movement enabled`,
        "clear @s",
        "stopsound @s fear.cs_fone"
    ];
    Promise.all(
        commands.map(command => 
            runcmd_at(player.name, command, true)
        )
    )
}



export async function toCommands(player) {
    const commands = [
        "clear @s",
        "playanimation @e[type=fear:start_button] animation.n.start_button.interact",
        "execute unless entity @a[tag=cs:lobby] run structure load mystructure:fireforks_red 72 44 49",
        "execute unless entity @a[tag=cs:lobby] run structure load mystructure:fireforks_red 68 44 49",
        "execute unless entity @a[tag=cs:lobby] run structure load mystructure:fireforks_red 66 44 49",
        "execute unless entity @a[tag=cs:lobby] run structure load mystructure:fireforks_red 62 44 49",
        `tellraw @a[tag=fear:lobby] {"rawtext":[{"translate":"fearcube.joined_lobby","with":{"rawtext":[{"selector":"@s"}]}}]}`,
        `tellraw @s {"rawtext":[{"translate":"fearcube.joined_you_lobby"}]}`
    ];

    for (const cmd of commands) {
        await runcmd_at(player.name, cmd, true);
    }

    
    player.setGameMode("adventure");
    playSound(player, "fear:fear_cube.hurt", 1, 1);
    playSound(player, "fear.cs_fone", 0.1, 1);
    ReplaceItemHotbar(player, "fear:selectteam", 0, 1, true);
    ReplaceItemHotbar(player, "fear:back", 8, 1, true);
    
    
    system.runTimeout(() => {
        const gameStatus = getScore('fear:settings', 'gameStatus', true);
        if (gameStatus === 1) {
            ReplaceItemHotbar(player, "fear:viewer", 4, 1, true);
        }
    }, 10);

}



function SelectTeam(player) {
    let teamT = getPlayerCountWithTag("lobby:terrorist");
    let teamC = getPlayerCountWithTag("lobby:commando");

    if (teamT < teamC) {
        player.addTag("lobby:terrorist");
    }
    else if (teamT > teamC) {
        player.addTag("lobby:commando")
    }
    else {
        player.addTag("lobby:terrorist")
    }
    system.runTimeout(() => {UiLobby()}, 7);
    
    
}

export function RemoveAllTags(player, lobby = "hub") {
    clearNonPlayerTags(player);


    system.runTimeout(() => {
        if (lobby === "lobby") {
            player.addTag("fear:lobby");
            SelectTeam(player);
            system.runTimeout(() => {startLobby();}, 1);
        }
    }, 10);
}


export function playerWatch(player) {
    const gameLocation = getScore('fear:settings', 'gameLocation', true);
    const gameStatus = getScore('fear:settings', 'gameStatus', true);
    if (gameLocation === 1 && gameStatus === 1) {
        cloceUI(player);
        removeTagsByPrefix(player, "lobby:");
        system.run(() => {
            player.teleport({ x: 1032, y: 138, z: -3 });
            player.addTag("fear:game");
            player.removeTag("fear:lobby");
            player.setGameMode("spectator");
            Tellraw(player, `==>§7`, "fearcube.lobby.command");
        });
        
    } else {
        runcmd_at(player.name, `clear @s fear:viewer`);

    } 
}