
import { setRaisingItems } from "../game/raisingItems.js";
import { giveBomb } from "../game/bomb.js";

import { getScore, setScore, runcmd_at, runcmd, Tellraw } from "../other/utils"; 
import { toLobby } from "../lobby/to_lobby.js"

const informationCommands = [
    "event entity @e[type=fear:informations] informations_skin0",
    "event entity @e[type=fear:informations] informations_skin1",
    "event entity @e[type=fear:informations] informations_skin2",
    "event entity @e[type=fear:informations] informations_skin3",
    "event entity @e[type=fear:informations] informations_skin4",
    "event entity @e[type=fear:informations] informations_skin5"
];

let currentCommandIndex = 0;

export function handleEntityInteraction(event) {
    const player = event.player;
    const name = player.name;
    const target = event.target;
    const tags = target.getTags();
    const entityName = tags.find(tag => tag.startsWith("DropItemID:"));
    const bombStatus = getScore("fear:settings", "bomb_condition", true);

    if (target.typeId === "fear:start_button") {
        const lockjoin = getScore('fear:settings', 'lockjoin', true);
        if (lockjoin === 0) {
            if (!player.hasTag(`fear:lobby`)) {
                toLobby(player);
            }
        } else {
            Tellraw(player, `§c`, "fearcube.message.join_blocked", " §6\\lockjoin");
            runcmd(`playanimation @e[type=fear:start_button] animation.n.start_button.interact`, true);
        }
        return;
    }

    if (target.typeId === "fear:back_lobby") {
        runcmd_at(name, `scriptevent fearcube:setLobby`, true);
        return;
    }

    // Информационная панель
    if (target.typeId === "fear:informations") {
        const cmd = informationCommands[currentCommandIndex];

        runcmd_at(name, `playanimation @e[type=fear:informations] animation.informations.click`, true);
        runcmd_at(name, `playsound random.pop @s ~~~ 0.5 0.5`, true);
        runcmd_at(name, cmd, true);

        currentCommandIndex = (currentCommandIndex + 1) % informationCommands.length;
        return;
    }

    // Бомба
    if (target.typeId === "fear:entites_bomb" && bombStatus === 0) {
        giveBomb(player);
        console.log("Interaction bomb")
        return;
    }
    // Голова
    if (target.typeId === "fear:head") {
        `"Interaction head" ${name}`
        return;
    }

    // Оружие и гранаты
    const weapons = {
        "fear:akm": "akm",
        "fear:awp": "awp",
        "fear:barrett": "barrett",
        "fear:m3": "m3",
        "fear:m249": "m249",
        "fear:mk18": "mk18",
        "fear:mp5": "mp5",
        "fear:p90": "p90",
        "fear:taurus": "taurus",
        "fear:fire_grenade": "fire_grenade",
        "fear:flash_grenade": "flash_grenade",
        "fear:he_grenade": "he_grenade",
        "fear:smoke_grenade": "smoke_grenade"
    };

    const itemId = weapons[target.typeId];
    if (itemId) {
        setRaisingItems(player, itemId, entityName);
    }
}
