import { getPlayerTagData, getTagValue, setDataTag, updateTagData } from '../other/tagUtils';
import { toLobby, setLobby } from '../lobby/to_lobby';
import { world, system } from "@minecraft/server";
import { saveOriginalNickname, restoreOriginalNickname } from '../other/playerNickName';
import { mapLaunch } from '../other/mapLaunch';
import { runcmd_at, removeTagsByPrefix, getRandomInRange, Tellraw } from "../other/utils.js";


export function handlePlayerJoin(event) {
    const playerName = event.playerName;
    let waitTicks = 0;

    const waitUntilReady = system.runInterval(() => {
        const player = world.getPlayers().find(p => p.name === playerName);
        waitTicks++;

        if (!player) return;
        if (!player.isValid) return;

        const pos = player.location;
        if (!pos || isNaN(pos.x)) return;

        system.clearRun(waitUntilReady);

        let ticks = 0;
        const intervalId = system.runInterval(() => {
            if (!player.isValid) {
                system.clearRun(intervalId);
                return;
            }

            runcmd_at(playerName, "clear @s");
            runcmd_at(playerName, "execute positioned 66 34 123 as @s[rm=1] at @s run tp @s 66 34 123 facing 68 33 109");

            ticks++;

            if (ticks >= 150) {
                system.clearRun(intervalId);
                try {
                    PlayerJoin(player);
                } catch (e) {}
            }
        }, 1);
    }, 1);
}



export function PlayerJoin(player) {
    const playerCount = world.getPlayers().length;
    const text = getRandomInRange(1, 3);
    if (playerCount === 1) {
        mapLaunch()
    }

    Tellraw(player, ``, "fearcube.greetings");
    Tellraw(player, `§7`);
    if (text === 1) {
        Tellraw(player, ``, "fearcube.help.prompt");
    } else {
        Tellraw(player, ``, "fearcube.help.command");
    } 

    if (!player.hasTag(`fearcube:fist`)) {
        setDataTag(player, "kills", 0);
        setDataTag(player, "deaths", 0);
        setDataTag(player, "playGames", 0);
        setDataTag(player, "scores", 0);

        system.run(() => {
            player.addTag(`fearcube:fist`);
        });

    }

    setLobby(player);
    removeTagsByPrefix(player, `"originalNick:`);
    saveOriginalNickname(player);
    player.triggerEvent("fear:custom_hitbox_off");
}
