import { system, world, Player } from '@minecraft/server';
import { 
    getScore, eventEntity,
    getPlayerCountWithTag, 
    playSound, Tellraw, 
    ActionBar, cloceUI,
    removeTagsByPrefix, 
    getPlayerNamesWithTag, 
    ReplaceItemHotbar, 
    getHealthPercentage, 
    ReplaceItem, getPlayerByName,
    hasItemInInventory,
    incrementScore,
    modifyScore,
    getRandomInRange,
    cloceForm

} from '../other/utils';

const originalNicknames = new Map();

function getOriginalNickname(player) {
    const tags = player.getTags();
    const originalNickTag = tags.find(tag => tag.startsWith("originalNick:"));
    return originalNickTag ? originalNickTag.replace("originalNick:", "") : player.name;
}


export function saveOriginalNickname(player) {
    system.run(() => {
        if (!originalNicknames.has(player.name)) {
            originalNicknames.set(player.name, player.nameTag);

            player.getTags()
                .filter(tag => tag.startsWith("originalNick:"))
                .forEach(tag => player.removeTag(tag));

            player.addTag(`originalNick:${player.nameTag}`);
        }
    });
}


export function setInvisibleNickname(player) {
    player.nameTag = "§7"; 
}

export function restoreOriginalNickname(player) {
    system.run(() => {
        const originalNickname = getOriginalNickname(player);
        player.nameTag = originalNickname;
        originalNicknames.delete(player.name);
    });
}





export function setRedNickname(player) {
    const originalNickname = getOriginalNickname(player);
    const balance = getScore("fear:balance", player, true);

    let bomb = "";
    if (hasBombInInventory(player)) {
        bomb = "";
    }
    player.nameTag = "§c" + originalNickname + `\n§a${balance}$ ${bomb}`; 
}

export function setBlueNickname(player) {
    const balance = getScore("fear:balance", player, true);
    const originalNickname = getOriginalNickname(player);
    player.nameTag = "§b" + originalNickname + `\n§a${balance}$`; 
}




function hasBombInInventory(player) {
    const inventory = player.getComponent("minecraft:inventory");
    if (!inventory) return false;

    const container = inventory.container;
    for (let i = 0; i < container.size; i++) {
        const item = container.getItem(i);
        if (item && item.typeId === "fear:bomb") {
            return true; 
        }
    }

    return false; 
}



system.afterEvents.scriptEventReceive.subscribe((event) => {
    const { id, sourceEntity, message } = event;

    if (!sourceEntity || typeof sourceEntity.name !== "string") return;

    const player = world.getPlayers().find(p => p.name === sourceEntity.name);
    if (!player) return;

    if (id === "fearcube:nickname" && message === "off") {
        setInvisibleNickname(player);
    } else if (id === "fearcube:nickname" && message === "on") {
        restoreOriginalNickname(player);
    } else if (id === "fearcube:nickname" && message === "red") {
        setRedNickname(player);
    } else if (id === "fearcube:nickname" && message === "blue") {
        setBlueNickname(player);
    } 
});


