import { Player, system } from '@minecraft/server';

const tagKey = "PlayerTag";

// Получить список
// const tags = getPlayerTagData(player);
// console.log(`Теги игрока: ${JSON.stringify(tags)}`);

// Добавление и удаление
// updateTagData(player, "kills", 1, true); // Добавляет +1 к тегу "kills"
// updateTagData(player, "deaths", 1, false); // Уменьшает значение тега "deaths" на 1


// Получение значения
// const kills = getTagValue(player, "kills");
// console.log(`Количество убийств: ${kills}`);

// Сет 
// setDataTag(player, "kills", 5); // Устанавливает ключ "kills" со значением 5

// Удаление
// setDataTag(player, "kills", null); // Удаляет ключ "kills"




function getTagData(tag) {
    try {
        return JSON.parse(tag);
    } catch (e) {
        return {};
    }
}

function setTagData(data) {
    return JSON.stringify(data);
}

function getPlayerTagData(player) {
    const currentTag = player.getTags().find(tag => tag.startsWith(tagKey));
    if (currentTag) {
        return getTagData(currentTag.replace(`${tagKey}=`, ''));
    }
    return {};
}

function getTagValue(player, key) {
    const tagData = getPlayerTagData(player);
    return tagData[key] !== undefined ? tagData[key] : null;
}

function setDataTag(player, key, value) {
    let tagData = {};
    const currentTag = player.getTags().find(tag => tag.startsWith(tagKey));

    if (currentTag) {
        tagData = getTagData(currentTag.replace(`${tagKey}=`, ''));
    }

    if (value === null || value === undefined) {
        delete tagData[key];
    } else {
        tagData[key] = value;
    }

    player.getTags()
        .filter(tag => tag.startsWith(`${tagKey}=`))
        .forEach(tag => player.removeTag(tag));

    player.addTag(`${tagKey}=${setTagData(tagData)}`);
}


function updateTagData(player, key, value, isAddition = true) {
    let tagData = {};
    const currentTag = player.getTags().find(tag => tag.startsWith(tagKey));

    if (currentTag) {
        tagData = getTagData(currentTag.replace(`${tagKey}=`, ''));
    }

    if (value === null) {
        delete tagData[key]; 
    } else {
        if (isAddition) {
            tagData[key] = (tagData[key] || 0) + value; 
        } else {
            tagData[key] = (tagData[key] || 0) - value; 
            if (tagData[key] <= 0) {
                delete tagData[key]; 
            }
        }
    }

    system.run(() => {
        player.getTags()
            .filter(tag => tag.startsWith(`${tagKey}=`))
            .forEach(tag => player.removeTag(tag));

        player.addTag(`${tagKey}=${setTagData(tagData)}`);
    });
}




system.afterEvents.scriptEventReceive.subscribe((event) => {
    const player = event.sourceEntity;

    if (player instanceof Player) {
        if (event.id === "fearcube:add_tag") {
            const [key, value] = event.message.split('=');
            updateTagData(player, key, parseInt(value), true);
        }

        if (event.id === "fearcube:remove_tag") {
            const [key, value] = event.message.split('=');
            updateTagData(player, key, parseInt(value), false);
        }

        if (event.id === "fearcube:list_tag") {
            const currentTag = player.getTags().find(tag => tag.startsWith(tagKey));
            if (currentTag) {
                const tagData = getTagData(currentTag.replace(`${tagKey}=`, ''));
                player.sendMessage(`Your tags: ${JSON.stringify(tagData)}`);
            } else {
                player.sendMessage("You have no tags.");
            }
        }

        if (event.id === "fearcube:reset_tag") {
            const key = event.message;
            updateTagData(player, key, null);
        }
    }
});


export { getPlayerTagData, getTagValue, setDataTag, updateTagData };
