import { system, world } from '@minecraft/server';
import { ActionFormData, MessageFormData, ModalFormData } from "@minecraft/server-ui";
import { shopItems, shopItemsPurchasing } from '../game/shopItems';
import { shopItemsitemID, dropItems } from '../game/raisingItems';
import { 
    getScore, 
    getPlayerCountWithTag, 
    playSound, Tellraw, 
    ActionBar, 
    removeTagsByPrefix, 
    getPlayerNamesWithTag, 
    ReplaceItemHotbar, 
    getHealthPercentage, 
    ReplaceItem,
    hasItemInInventory,
    incrementScore,
    modifyScore

} from '../other/utils';



export function ShopGameMain(player) {
    const balance = getScore('fear:balance', player, true);
    const form = new ActionFormData();
    form.title("%trial.tabbed_upsell.store.title");

    const isTerrorist = player.hasTag('game:terrorist');

    shopItems.forEach(item => {
        const price = isTerrorist ? item.priceT : item.priceC;
        const priceColor = balance >= price ? '§l§q' : '§l§m';
        const formattedPrice = `${priceColor}${price}$`;

        
        form.button(`${formattedPrice}`, `textures/fearcube/gun_icon/${item.name}`);
    });

    form.show(player).then(r => {
        if (r.canceled) return;

        const response = r.selection;
        if (response !== undefined && response < shopItems.length) {
            const selectedItem = shopItems[response];

            
            if (isTerrorist && selectedItem.name === "sapper_kit") {
                Tellraw(player, `§c`, "terrorists.no_defuse");
                return;
            }

            purchasingItem(player, response);
            ShopGameMain(player);
        }
    }).catch(e => {
        console.error(e, e.stack);
    });
}



export function purchasingItem(player, id) {
    const item = shopItemsitemID.find(i => i.ID === id.toString());
    if (!item) {
        return;
    }

    const shopItem = shopItems.find(i => i.ID === id.toString());
    if (!shopItem) {
        return;
    }
    const balance = getScore('fear:balance', player, true);
    const price = player.hasTag('game:terrorist') 
        ? parseInt(shopItem.priceT || 0) 
        : parseInt(shopItem.priceC || 0);

    if (balance >= price) {
        const parsedPrice = parseInt(price, 10);
        modifyScore("fear:balance", `${player.name}`, -parsedPrice);

        setRaisingItems1(player, item.name);
        Tellraw(player, "§7==> ", "fearcube.shop.success", "§a ", item.textRaising);
    } else {
        playSound(player, "note.bass", 1, 1);
        Tellraw(player, "§c", "fearcube.shop.error.nomoney", "§l ", item.textRaising);
    }
}




function setRaisingItems1(player, name) {
    const itemConfig = shopItemsitemID.find(item => item.name === name);

    if (!itemConfig) {
        return;
    }

    const currentSlotItem = getItemInSlot(player, itemConfig.slotID, itemConfig.slot);
    if (currentSlotItem) {
        const existingItemConfig = shopItemsitemID.find(item => 
            item.itemID === currentSlotItem || 
            item.alternative1 === currentSlotItem || 
            item.alternative2 === currentSlotItem
        );
        
        const existingItemName = existingItemConfig ? existingItemConfig.name : currentSlotItem;

        if (itemConfig.itemID !== "minecraft:netherite_chestplate") {
            dropItems(player, existingItemName);
        }        
    }

    ReplaceItem(
        player,
        itemConfig.itemID,
        itemConfig.slotID,
        itemConfig.slot,
        parseInt(itemConfig.quantity || 1),
        true
    );

    if (itemConfig.itemID1 && itemConfig.slot1 !== undefined) {
        ReplaceItem(
            player,
            itemConfig.itemID1,
            itemConfig.slotID1,
            itemConfig.slot1,
            parseInt(itemConfig.quantity1 || 1),
            true
        );
    }

    playSound(player, "fear.taking_up_arms", 1, 1);
}



function getItemInSlot(player, slotID, slot) {
    try {
        const inventory = player.getComponent("minecraft:inventory")?.container;
        if (!inventory) {
            return null;
        }

        const itemStack = inventory.getItem(parseInt(slot));
        return itemStack ? itemStack.typeId : null;
    } catch (error) {
        console.error(`Ошибка получения предмета из слота ${slotID}:${slot} - ${error.message}`);
        return null;
    }
}