

import { world, system, Player, EntityHealthComponent, EntityComponentTypes  } from "@minecraft/server";
import { getScore, getPlayerCountWithTag, playSound, Tellraw, ActionBar, removeTagsByPrefix, getPlayerNamesWithTag, ReplaceItemHotbar, getHealthPercentage, runcmd_at, runcmd } from '../other/utils';


function UIglobalGame(player) {
    const players = world.getPlayers();

    const iconTerrorist = "";
    const iconCommando = "";
    const iconDeathTerrorist = "";
    const iconDeathCommando = "";
    const iconAir = "";

    const terroristIcons = players
        .filter(p => p.getTags().some(tag => tag.startsWith("terrorist:")))
        .map(p => (p.getGameMode() === "spectator" ? iconDeathTerrorist : iconTerrorist))
        .join("");

    const commandoIcons = players
        .filter(p => p.getTags().some(tag => tag.startsWith("commando:")))
        .map(p => (p.getGameMode() === "spectator" ? iconDeathCommando : iconCommando))
        .join("");

    const maxCount = Math.max(terroristIcons.length, commandoIcons.length);
    const balancedTerroristIcons = iconAir.repeat(maxCount - terroristIcons.length) + terroristIcons;
    const balancedCommandoIcons = commandoIcons + iconAir.repeat(maxCount - commandoIcons.length);

    const displayText = `${balancedTerroristIcons}     ${balancedCommandoIcons}`;

    runcmd(`title @a[tag=fear:game] title bossbar${displayText}`, true);
}



function UItopSubtitleGame(player, safeDisplayText1 = "123", safeDisplayText2 = "") {
    const players = world.getPlayers();

    const terroristNames = players
        .filter(p => p.getTags().some(tag => tag.startsWith("terrorist:")))
        .map(p => p.name.slice(0, 5).padEnd(5, " "));

    const commandoNames = players
        .filter(p => p.getTags().some(tag => tag.startsWith("commando:")))
        .map(p => p.name.slice(0, 5).padEnd(5, " "));

    const indentation = "    ";
    const corectInterval = "       ";

    const maxCount = Math.max(terroristNames.length, commandoNames.length);

    const balancedTerroristNames = [
        ...Array(maxCount - terroristNames.length).fill(corectInterval),
        ...terroristNames
    ].join(indentation);

    const balancedCommandoNames = [
        ...commandoNames,
        ...Array(maxCount - commandoNames.length).fill(corectInterval)
    ].join(indentation);

    const displayText = `§c${balancedTerroristNames}                        §b${balancedCommandoNames}`;
    const safeDisplayText = displayText.replace(/"/g, '\\"');

    const timer = getScore('fear:settings', 'purchaseTimer', true);

    runcmd(`title @a[tag=fear:game] title top_subtitle4"${safeDisplayText}"`, true);

    if (!player.hasTag(`game:terrorist`) && !player.hasTag(`game:commando`)) {
        
        const gameStatus = getScore('fear:settings', 'gameStatus', true);
        if (gameStatus === 1) {
            runcmd_at(player.name, `execute positioned 1032 138 -3 as @s[rm=90] at @s run spreadplayers 1032 -3 1 30 @s`, true);
        }

    }


    
    system.runTimeout(() => {
        if (safeDisplayText1 && safeDisplayText1.trim() !== "") {
            const name = player.name;
            runcmd_at(name, `title @s title top_subtitle2"${safeDisplayText1}"`, true);
        }

        system.runTimeout(() => {
            if (timer > 1 && safeDisplayText2 && safeDisplayText2.trim() !== "") {
                runcmd_at(player.name, `title @s title top_subtitle3"${safeDisplayText2}"`, true);
            } else {
                runcmd(`title @a[tag=fear:game] title top_subtitle3`, true);
            }
            system.runTimeout(() => {
                const spectators = world.getPlayers().filter(player => {
                    const tags = new Set(player.getTags());
                    return tags.has("fear:game") &&
                        !tags.has("game:terrorist") &&
                        !tags.has("game:commando");
                });

                const viewer = spectators.length >= 1 ? ` §c${spectators.length}` : "";

                players.forEach(p => {
                    const balance = getScore('fear:balance', p, true);
                    
                    if (!p.hasTag("game:terrorist") || !p.hasTag("game:commando")) {
                        const name = p.name;
                        runcmd_at(name, `title @s title upper_right_corner"${viewer}"`, true);
                    }
                    if (p.hasTag("game:terrorist") || p.hasTag("game:commando")) {
                        const name = p.name;
                        runcmd_at(name, `title @s title upper_right_corner"§a${balance}$${viewer}"`, true);
                    }
                });

            }, 1);
        }, 1);

    }, 1);
}




function UItopSubtitle2Game(player, safeDisplayText2) {
    const players = world.getPlayers();

    const smaillHead = "";
    const smaillDeath = "    ";
    const replacementNick = "       ";

    const isPlayerTerrorist = player.getTags().some(tag => tag.startsWith("terrorist:"));
    const isPlayerCommando = player.getTags().some(tag => tag.startsWith("commando:"));

    const getFormattedHealth = (currentPlayer) => {
        if (currentPlayer.getGameMode() === "spectator") {
            return `${smaillDeath}`;
        }
        if ((isPlayerTerrorist && currentPlayer.getTags().some(tag => tag.startsWith("commando:"))) ||
            (isPlayerCommando && currentPlayer.getTags().some(tag => tag.startsWith("terrorist:")))) {
            return `${replacementNick}`;
        }
        const healthPercentage = getHealthPercentage(currentPlayer);
        return `${healthPercentage}${smaillHead}`;
    };

    const terroristHealth = players
        .filter(p => p.getTags().some(tag => tag.startsWith("terrorist:")))
        .map(p => getFormattedHealth(p));

    const commandoHealth = players
        .filter(p => p.getTags().some(tag => tag.startsWith("commando:")))
        .map(p => getFormattedHealth(p));

    const indentation = "    ";
    const corectInterval = "       ";

    const maxCount = Math.max(terroristHealth.length, commandoHealth.length);

    const balancedTerroristHealth = [
        ...Array(maxCount - terroristHealth.length).fill(corectInterval),
        ...terroristHealth
    ].join(indentation);

    const balancedCommandoHealth = [
        ...commandoHealth,
        ...Array(maxCount - commandoHealth.length).fill(corectInterval)
    ].join(indentation);

    const displayText = `${balancedTerroristHealth}                          ${balancedCommandoHealth}`;
    const safeDisplayText1 = displayText.replace(/"/g, '\\"');

    UItopSubtitleGame(player, safeDisplayText1, safeDisplayText2);
}

function UItopSubtitle3Game(player, safeDisplayText2) {
    const smaillScoreKill = ""; 
    const smaillScoreDeath = ""; 

    const players = world.getPlayers();

    const terroristStats = players
        .filter(p => p.getTags().some(tag => tag.startsWith("terrorist:")))
        .map(p => {
            const kills = getScore("fear:kill", p, true);
            const deaths = getScore("fear:deach", p, true);
            return `${smaillScoreKill}${kills} ${smaillScoreDeath}${deaths}`;
        });
    const commandoStats = players
        .filter(p => p.getTags().some(tag => tag.startsWith("commando:")))
        .map(p => {
            const kills = getScore("fear:kill", p, true);
            const deaths = getScore("fear:deach", p, true);
            return `${smaillScoreKill}${kills} ${smaillScoreDeath}${deaths}`;
        });

    const indentation = "   ";
    const corectInterval = "        ";
    const maxCount = Math.max(terroristStats.length, commandoStats.length);

    const balancedTerroristStats = [
        ...Array(maxCount - terroristStats.length).fill(corectInterval),
        ...terroristStats
    ].join(indentation);

    const balancedCommandoStats = [
        ...commandoStats,
        ...Array(maxCount - commandoStats.length).fill(corectInterval)
    ].join(indentation);
 
    const displayText = `${balancedTerroristStats}                      ${balancedCommandoStats}`;
    const safeDisplayText1 = displayText.replace(/"/g, '\\"');
    UItopSubtitleGame(player, safeDisplayText1, safeDisplayText2);
}


function UItopSubtitle4Game(player) {
    const players = world.getPlayers();
    const maxBalanceLength = 5;

    const isPlayerTerrorist = player.getTags().some(tag => tag.startsWith("terrorist:"));
    const isPlayerCommando = player.getTags().some(tag => tag.startsWith("commando:"));

    const getFormattedBalance = (currentPlayer) => {
        const isEnemy = (isPlayerTerrorist && currentPlayer.getTags().some(tag => tag.startsWith("commando:"))) ||
                        (isPlayerCommando && currentPlayer.getTags().some(tag => tag.startsWith("terrorist:")));
        if (isEnemy) {
            return "       "; 
        }
        const balance = getScore("fear:balance", currentPlayer, true);
        const formattedBalance = balance.toString().padStart(maxBalanceLength, " ");
        return `§a${formattedBalance}\$§r`;
    };

    const terroristStats = players
        .filter(p => p.getTags().some(tag => tag.startsWith("terrorist:")))
        .map(p => getFormattedBalance(p));

    const commandoStats = players
        .filter(p => p.getTags().some(tag => tag.startsWith("commando:")))
        .map(p => getFormattedBalance(p));

    const indentation = "   ";
    const corectInterval = "       ";

    const maxCount = Math.max(terroristStats.length, commandoStats.length);
    const balancedTerroristStats = [
        ...Array(maxCount - terroristStats.length).fill(corectInterval),
        ...terroristStats
    ].join(indentation);

    const balancedCommandoStats = [
        ...commandoStats,
        ...Array(maxCount - commandoStats.length).fill(corectInterval)
    ].join(indentation);

    const displayText = `${balancedTerroristStats}                        ${balancedCommandoStats}`;
    const safeDisplayText2 = displayText.replace(/"/g, '\\"');

    const timer = getScore('fear:settings', 'purchaseTimer', true);
    if (timer > 1) {
        UItopSubtitle3Game(player, safeDisplayText2); 
    } else if (timer === 0) {
        UItopSubtitle2Game(player); 
    }
}

const bombTickCounter = new Map();
const ANIMATION_SPEED = 2;

export function UItopTimer(player) {
    const timer1 = getScore("fear:settings", "purchaseTimer", true) || 0;
    const timer2 = getScore("fear:settings", "startTimerSpecialForces", true) || 0;

    let timer = Math.max(timer1, timer2);

    const roundsTakenTerrorist = getScore("fear:settings", "roundsTakenTerrorist", true) || 0;
    const roundsTakenCommando = getScore("fear:settings", "roundsTakenCommando", true) || 0;

    let letWT = "§e";
    let letWC = "§e";

    if (roundsTakenTerrorist < roundsTakenCommando) {
        letWT = "§c";
        letWC = "§a";
    } else if (roundsTakenTerrorist > roundsTakenCommando) {
        letWT = "§a";
        letWC = "§c";
    }

    const minutes = Math.floor(timer / 60);
    const seconds = (timer % 60).toString().padStart(2, "0");

    const bombStatus = getScore("fear:settings", "bomb_condition", true);
    let formattedTimer;

    if (bombStatus === 0) {
        formattedTimer = `${minutes}:${seconds}`;
        bombTickCounter.set(player.id, 0);
    } else if (bombStatus === 1) {
        let tick = bombTickCounter.get(player.id) || 0;
        tick++;
        bombTickCounter.set(player.id, tick);

        
        formattedTimer = tick % (ANIMATION_SPEED * 2) < ANIMATION_SPEED ? "" : "";
    } else if (bombStatus === 2) {
        formattedTimer = "";
    }

    const displayText = `§6${formattedTimer}\n§r§a${letWT}${roundsTakenTerrorist} §6| ${letWC}${roundsTakenCommando}`;
    const safeDisplayText = displayText.replace(/"/g, '\\"');

    runcmd(`title @a[tag=fear:game] title timer_top_midle${safeDisplayText}`, true);
}




export function UIupdateGame() {
    const players = world.getPlayers().filter(player => player.hasTag("fear:game"));

    players.forEach(player => {
        system.runTimeout(() => {
            UIglobalGame(player);
            system.runTimeout(() => {
                UItopTimer(player)
                system.runTimeout(() => {
                    UItopSubtitle4Game(player)
                }, 2);
            }, 2);
        }, 1);
    });
}


export function clearLobbyUI(player) {
    system.runTimeout(() => { 
        runcmd_at(player.name, `title @s title linen_plate`, true);
        system.runTimeout(() => { 
            runcmd_at(player.name, `title @s title scoreboard`, true);
            system.runTimeout(() => { 
                runcmd_at(player.name, `title @s title big_actionbar`, true);
            }, 1);
        }, 1);
    }, 1);
}