import { getPlayerTagData, getTagValue, setDataTag, updateTagData } from '../other/tagUtils';
import { getTimerValue, startLobby } from '../lobby/timer';
import { getScore, getPlayerCountWithTag, playSound, Tellraw, ActionBar, removeTagsByPrefix, getPlayerNamesWithTag, runcmd, runcmd_at } from '../other/utils';

import { world, system, Player } from "@minecraft/server";

export function UIglobalLobby(player) {
    const kills = getTagValue(player, "kills");
    const deaths = getTagValue(player, "deaths");
    const playGames = getTagValue(player, "playGames");
    const scores = getTagValue(player, "scores");
    const remainingTime = getTimerValue();

    let nameUI = `,{"text":" §7§lCounter-Strike"}`;

    let timer = "";
    if (remainingTime > 0) {timer = `,{"text":"\n\n §r"},{"translate":"fearcube.timer.name"},{"text":":§6 ${remainingTime}"}`}


    let team = "";
    let teamL = "";
    if (player.hasTag(`lobby:terrorist`)) { team = "fearcube.team.terrorists"; teamL = "§m ";}
    if (player.hasTag(`lobby:commando`)) { team = "fearcube.team.specialforces"; teamL = "§b ";}
    let teamInfo = `,{"text":"\n\n §r"},{"translate":"fearcube.team.side.yours"},{"text":":\n   ${teamL}"},{"translate":"${team}"}`;

    let kdr;
    if (deaths === 0) { kdr = kills; } else { kdr = kills / deaths; }
    let kda = `,{"text":"\n§r"},{"translate":"fearcube.coefficient.name"},{"text":":§a ${kdr.toFixed(1)}"}`;

    
    let playG = `,{"text":"\n\n§r"},{"translate":"fearcube.total_matches.name"},{"text":":§a ${playGames}"}`;
    let score = `,{"text":"\n§r"},{"translate":"fearcube.earned_points.name"},{"text":":§a ${scores}"}`;
    let website = `,{"text":"\n\n §bwww.fearcube.com"}`;

    const name = player.name;
    runcmd_at(name, `title @s title timer_top_midle`, true);

    const rawTextJSON = `{"rawtext":[{"text":"scoreboard"}${nameUI}${timer}${playG}${score}${kda}${teamInfo}${website}]}`;
    runcmd_at(name, `titleraw @s title ${rawTextJSON}`, true);
    
}

export function UItopCenterLobby(player) {

    const countTerrorist = getPlayerCountWithTag("lobby:terrorist");
    const countCommando = getPlayerCountWithTag("lobby:commando");

    const iconTerrorist = "";
    const iconCommando = "";
    const iconAir = "";

    const maxCount = Math.max(countTerrorist, countCommando);
    const paddedTerrorist = iconAir.repeat(maxCount - countTerrorist) + iconTerrorist.repeat(countTerrorist);
    const paddedCommando = iconCommando.repeat(countCommando) + iconAir.repeat(maxCount - countCommando);

    const displayText = `${paddedTerrorist} vs ${paddedCommando}`;

    const name = player.name;
    clearGameUI(player);
    runcmd_at(name, `title @s title bossbar${displayText}`, true);

}



export function UItopSubtitleLobby(player) {
    const players = world.getPlayers();
    const terroristNames = players
        .filter(player => player.hasTag("lobby:terrorist"))
        .map(player => {
            const name = player.name.slice(0, 5);
            return player.name.length > 5 ? `${name}` : name;
        });

    const commandoNames = players
        .filter(player => player.hasTag("lobby:commando"))
        .map(player => {
            const name = player.name.slice(0, 5);
            return player.name.length > 5 ? `${name}` : name;
        });

    const indentation = "    ";
    const corectInterval = "      ";

    const maxCount = Math.max(terroristNames.length, commandoNames.length);

    const balancedTerroristNames = [
        ...Array(maxCount - terroristNames.length).fill(corectInterval),
        ...terroristNames
    ].join(indentation);
    

    const balancedCommandoNames = [
        ...commandoNames,
        ...Array(maxCount - commandoNames.length).fill(corectInterval)
    ].join(indentation);

    const displayText = `  §m${balancedTerroristNames}           §r|§b              ${balancedCommandoNames}`;
    const safeDisplayText = displayText.replace(/"/g, '\\"');

    const name = player.name;
    runcmd_at(name, `title @s title linen_plate${safeDisplayText}`, true);
}

export function UiLobby() {
    const players = world.getPlayers().filter(player => player.hasTag("fear:lobby"));

    players.forEach(player => {
        UItopCenterLobby(player);
        system.runTimeout(() => {
            UItopSubtitleLobby(player);
            system.runTimeout(() => {
                UIglobalLobby(player);
                clearGameUI(player);  
            }, 1);
        }, 1);   
    });
}



function clearGameUI(player) {
    system.runTimeout(() => { 
        runcmd_at(player.name, `title @s title top_subtitle3`, true);
        system.runTimeout(() => { 
            runcmd_at(player.name, `title @s title top_subtitle2`, true);
            system.runTimeout(() => { 
                runcmd_at(player.name, `title @s title upper_right_corner`, true);
                system.runTimeout(() => { 
                    runcmd_at(player.name, `title @s title timer_top_midle`, true);
                    system.runTimeout(() => { 
                        runcmd_at(player.name, `title @s title top_subtitle4`, true);
                    }, 1);
                }, 1);
            }, 1);
        }, 1);
    }, 1);
}

